<?php 

class DokuOAuthServer extends OAuthServer {/*{{{*/
    
    public function get_dokuwiki_user($consumer_key, $token_key) {/*{{{*/
        require("tokens.php");
        foreach($oauth_tokens as $t) {
            if (   ($t['key']   == $consumer_key)
                && ($t['token'] == $token_key)) 
                return $t['user'];
        }
        return NULL;
    }/*}}}*/

}/*}}}*/

class DokuOAuthDataStore extends OAuthDataStore {/*{{{*/
    private $dbh;

    function __construct($path = '') {/*{{{*/ 
        $this->dbh = dba_popen($path, 'c', 'inifile');
    }/*}}}*/

    function __destruct() {/*{{{*/
        dba_close($this->dbh);
    }/*}}}*/

    function lookup_consumer($consumer_key) {/*{{{*/
        require("tokens.php");
        foreach($oauth_tokens as $t) {
            if($t['key'] == $consumer_key) {
                return new OAuthConsumer($t['key'], $t['secret'], NULL);
            }
        }
        return NULL;
    }/*}}}*/

    function lookup_token($consumer, $token_type, $token) {/*{{{*/
        if ($token_type != 'access') return NULL;
        require("tokens.php");
        foreach($oauth_tokens as $t) {
            if ($consumer->key == $t['key']
                && $token == $t['token']) {
                return new OAuthToken($t['token'], $t['token_secret'], 1);
            }
        }
        return NULL;
    }/*}}}*/

    function cleanup_nonce() {/*{{{*/
        $key = dba_firstkey($this->dbh);
        $now=time();
        $handle_later = array();
        while ($key != false) {
        # clean up old-NONCEs -> oauth_timestamp ;; OAuthServer->timestamp_threshold (300 sec)
        # give it some extra hour(s) eg. server changes time-zones...
            if (!strncmp($key,"nonce_",6) && dba_fetch($key, $this->dbh) + 3900 < $now) 
                $handle_later[] = $key;
        # same for old session-tokens.
            if (!strncmp($key,"session_",8) && $rv=dba_fetch($key, $this->dbh)) {
                $data=unserialize($rv);
                if ($data['created'] + 3900 < $now) 
                    $handle_later[] = $key;
            }
            $key = dba_nextkey($this->dbh);
        }
        foreach ($handle_later as $key) {
            dba_delete($key, $this->dbh);
        }
    }/*}}}*/

    function lookup_nonce($consumer, $token, $nonce, $timestamp) {/*{{{*/
        if (dba_exists("nonce_$nonce", $this->dbh)) {
            return TRUE;
        } else {
            dba_insert("nonce_$nonce", time(), $this->dbh);
            $this->cleanup_nonce();
            return FALSE;
        }
    }/*}}}*/

}/*}}}*/

/* vim: set ts=4 sw=4 et foldmethod=marker enc=utf-8 : */
