<?php
/**
 * @license    GPL 2 (http://www.gnu.org/licenses/gpl.html)
 * @author     Robin Gareus <robin@gareus.org>
 */

// must be run within Dokuwiki
if(!defined('DOKU_INC')) die();

class action_plugin_oauth extends DokuWiki_Action_Plugin {

    var $_outargs = NULL;

    /**
     * return some info
     */
    function getInfo(){
        return array(
            'author' => 'Robin Gareus',
            'email'  => 'robin@gareus.org',
            'date'   => '2009-03-12',
            'name'   => 'oauth',
            'desc'   => 'Authorize User requests by oAuth - Simple Version',
            'url'    => 'http://rg42.org/wiki/dokuoauth_simple',
        );
    }

    /**
     * register the eventhandlers
     */
    function register(&$contr){/*{{{*/
        $contr->register_hook('ACTION_ON_AUTH',
                              'BEFORE',
                              $this,
                              'handle_act_authhook');

        $contr->register_hook('OUTGOING_HTTP_REQUEST',
                              'BEFORE',
                              $this,
                              'handle_sign_request');
    }/*}}}*/

    /**
     * handles oauth-signed requests
     * 
     * validates signature and sets $_SERVER['REMOTE_USER']
     * and global $USERINFO  just like auth_login(). 
     */
    function handle_act_authhook(&$event, $param){/*{{{*/
        if (is_array($_REQUEST['do']) && !empty($_REQUEST['do']['oauth'])) return; // skip requests to oauth-API

        if (!empty($_REQUEST['oauth_signature'])) {
            require_once("dokuoauth.php");
            $user='';
            // verify signature - check consumer and access token
            try {
                $req = OAuthRequest::from_request();
                list($consumer, $token) = $doku_server->verify_request($req);
                $user=$doku_server->get_dokuwiki_user($consumer->key, $token->key);

            } catch (OAuthException $e) {
                print($e->getMessage() . "\n<hr />\n");
                print_r($req);
                $this->_debug("oauth error:\n".$e->getMessage()."\n".print_r($req, true));
                die();
            }
            if (!$this->getConf('enable')) {
                $this->_debug('oAuth: good signature but plugin is not enabled.');
                msg('good signature - but OAuth authorization not enabled in plugin configuration');
                return;
            }

            if (empty($user)) $this->_debug('oAuth: empty user name.');
            if (empty($user)) {
                $this->_debug("signature ok, mapped username is empty.");
                return;
            }

            // -> set username for this session
            $_SERVER['REMOTE_USER'] = $user;
            global $USERINFO;
            global $auth;
            $USERINFO = $auth->getUserData($user);
            if (!is_array($USERINFO)) {
                $_SERVER['REMOTE_USER'] = ""; // after _log() ?
                $this->_debug('oAuth: could not find user: '.$user);
            } 
            else { 
                $this->_debug('oAuth: set-user: '.$user);
                # Authorized User - skip auth_login() 
                #$event->stopPropagation();
                $event->preventDefault();
            }
        }
    }/*}}}*/

    /**
     * called for each outgoing HTTP request from
     * inc/HTTPClient.php
     * 
     * sign the request is possible.
     */
    function handle_sign_request(&$event){/*{{{*/
        if (!$this->getConf('enable')) {
            $this->_debug('not signing request - oAuth plugin is disabled.');
            $event->preventDefault();
            return true;
        }

        $uri  = parse_url($event->data['url']);
        $path = $uri['path'];
        parse_str($uri['query'],$params);

        if ($event->data['method'] == 'POST') { 
            # only for $headers["Content-Type"] == "application/x-www-form-urlencoded"
            if (is_array($event->data['data'])) {
                parse_str($event->data['data'], $postparams);
                $params=array_merge($params,$postparams);
            } else {
                $this->_debug('oAuth signing non www-form-urlencoded POSTS is not supported');
                $event->preventDefault();
                return true;
            }
        }

        // skip if there is already an oauth_* signature
        if (in_array("oauth_nonce", array_keys($params))) { 
            $event->preventDefault();
            return true;
        }

        global $doku_oauth_consumer;
        $doku_oauth_consumer= true;
        require("dokuoauth.php");

        $oak=NULL;
        global $oa_providers;
        foreach ($oa_providers as $p) { 
            if ($p['host'] != $uri['host']) continue;
            if (!empty($p['user']) && $p['user'] != $_SERVER['REMOTE_USER']) continue;
            $oak = $p;
            break;
        }

        if (!is_array($oak)) {
            $this->_debug('no tokens avail to sign this request');
            $event->preventDefault();
            return true;
        }

        try {
            $sig_method = new OAuthSignatureMethod_HMAC_SHA1(); 
            $consumer = new OAuthConsumer($oak['key'], $oak['secret'], NULL);
            $token = new OAuthToken($oak['token'], $oak['token_secret']);
            $req = OAuthRequest::from_consumer_and_token($consumer, $token, $event->data['method'], $uri['scheme']."://".$uri['host'].$uri['path'], $params);
            #$this->_debug('base-string: '.print_r($req->get_signature_base_string(),true));
            $req->sign_request($sig_method, $consumer, $token);
        } catch (OAuthException $e) {
            $this->_debug("oauth error:\n".$e->getMessage()."\n".print_r($req, true));
            $event->preventDefault();
            return;
        }

        $this->_debug('signed-url: '.print_r($req->to_url(),true));

        if ($event->data['method'] == "POST") {
            $event->data['url']  = $req->get_normalized_http_url();
            $event->data['data'] = $req->to_postdata();
        }  else {
            $event->data['url']  = $req->to_url();
            $event->data['data'] = '';
        }
        return true;

    }/*}}}*/

    private function _debug ($m = null){ /*{{{*/
        $OAdebug= $this->getConf('trace_plugin'); 
        $OAlogfile= '/tmp/oAuth.debug';
        if (! isset($OAdebug) || $OAdebug === false) return;

        if (! is_writable(dirname($OAlogfile)) &! is_writable($OAlogfile)){
            header("HTTP/1.1 500 Internal Server Error");
            echo 'Cannot write to debug log: ' . $OAlogfile;
            return;
        }
        $vhost=DOKU_URL;
        error_log($vhost.' '.date("c ").$m."\n", 3, $OAlogfile);
    } /*}}}*/

}

/* vim: set ts=4 sw=4 et foldmethod=marker enc=utf-8 : */
